/* --------------------------------------------------------------
    Signed integers with unlimited range (version 1.1a).

    http://www.imach.uran.ru/cbignum

    Copyright 1999-2005 by Raul N.Shakirov, IMach of RAS(UB).
    All Rights Reserved.

    Permission has been granted to copy, distribute and modify
    software in any context without fee, including a commercial
    application, provided that the aforesaid copyright statement
    is present here as well as exhaustive description of changes.

    THE SOFTWARE IS DISTRIBUTED "AS IS". NO WARRANTY OF ANY KIND
    IS EXPRESSED OR IMPLIED. YOU USE AT YOUR OWN RISK. THE AUTHOR
    WILL NOT BE LIABLE FOR DATA LOSS, DAMAGES, LOSS OF PROFITS OR
    ANY OTHER KIND OF LOSS WHILE USING OR MISUSING THIS SOFTWARE.
-------------------------------------------------------------- */
#ifndef _CBIGNUM_H
#define _CBIGNUM_H

#ifndef __cplusplus
#error  Must use C++ for the type cBigNumber.
#endif

#include <iostream.h>

#ifndef EXARRAY_H
#include "Exarray.h"
#include "Exdebug.h"
#endif//EXARRAY_H

//================================================
//      Internal representation of cBigNumber.
//================================================
//      Unlimited number is stored in array of type long.
//      Word 0 contains number of long words, required for
//      storing of complementary code of number, the next
//      words contain complementary code in order from low
//      words to high words. Sign resides in the last word.
//      Method length() tells number of words in the code.
//
//      Minimal number of long word are used for storing,
//      but no less then one word:
//      - if length()>1, high word differs from sign extension
//        of word preceding the high;
//      - zero is represented by single 0-word.
//      We'll call this representation to be normalized.
//
//      Normalization is not mandatory. Methods allow for any
//      number of words in the complementary code, starting from 0.
//      Obtaining of normalized result is guaranteed if all
//      operands are normalized.
//
//      From managing of long array we use class exblock<long>,
//      which object contains reference to array with code of number
//      and also field with overall number of long words, available
//      for recording if number. Number of available words may be
//      be greater then number of occupied words as this is necessary
//      for arithmetical operation, which may require for increasing
//      of size of number.
//
//      Resizing of array is carried out by means of the following
//      methods:
//      checkexpand(n) - increase number of available words and
//                       reserve some memory for further expansion.
//      checkindex(n)  - decrease number of available words but
//                       do not free reserved memory.
//      Here number of words does not include word with index 0.
//      Note that array may reserve more words than it is required.
//
//      If macro NCHECKPTR is defined, method checkexpand() increases
//      number of available words uo to number of actually reserved
//      words, whereas method checkindex() does nothing.
//
//      If macro NCHECKPTR is not defined, then methods checkexpand()
//      and checkindex() sets number of available words to be exactly
//      same as indicated.
//
//      Hierarchy of classes:
//
//      exblock<long>           // Array with variable length.
//      |
//      +- cBigNumber           // Unlimited number.
//         |
//         +- cBigTemp          // Unlimited number in the stack.
//            |
//            +- cBigInc        // Postfix increment.
//            +- cBigDec        // Postfix decrement.
//            +- cBigPos        // Unary +.
//            +- cBigNeg        // Unary -
//            +- cBigCompl      // Bit-wise inversion.
//            +- cBigXor        // Bit-wise sum by module 2.
//            +- cBigAnd        // Bit-wise conjunction.
//            +- cBigOr         // Bit-wise disjunction.
//            +- cBigAdd        // Addition.
//            +- cBigSub        // Subtraction.
//            +- cBigMul        // Multiplication.
//            +- cBigDiv        // Division.
//            +- cBigMod        // Module.
//            +- cBigShl        // Left shift.
//            +- cBigShr        // Right shift.
//            +- cBigPow        // Power.
//            +- cBigPowMod     // Power by module.
//            +- cBigSqrt       // Square root.

//================================================
//      Main class cBigNumber.
//================================================

class cBigNumber:       public exblock<long>
{
protected:                      // Optimized method for creating of number
                                // with minimal size and given initial value.
  void  create (long b = 0)     // Called from some constructors instead of
  {                             // checkexpand (1).
    e = (long *) exmalloc (EXCALCBLOCKSIZE_1);
    e [0] = 1; e [1] = b;
    len = max_size (EXCALCBLOCKSIZE_1);
    checkindex (1);
  }
                                        // Constructor for cBigTemp.
  cBigNumber (exblockstack<long> &stack) { stack.get(*this); }

public:

//      Constructors.                                   // Initial value:
  cBigNumber ()                 { create (0);   }       // Zero.
  cBigNumber (long b)           { create (b);   }       // Number.
  cBigNumber (const cBigNumber&);                       // Unlimited number.
  cBigNumber (const char *psz, unsigned radix);         // String.

//      Destructor.
  ~cBigNumber()                 { exfree (e);   }

//      Information on number.
  size_t      length () const   { return *(size_t*)e; } // Number of words.
  const long* code   () const   { return e + 1;       } // Array of words.
  const long& loword () const   { return e [1];       } // Low word.
  const long& hiword () const   { return e [length()];} // High word.
  void        dump   () const;                          // Dump array of words.
  void        erange () const;                          // Range error.
  int         isfit  () const;                          // Normalized?
  int         bits   () const;                          // Num of valid bits.

//      Comparison operations. Returns -1 = less, 0 = equal, 1 = greater.
  int         comp  (const cBigNumber&) const;          // Compare to
  int         comp  (long)              const;          // given word.
  int         comp0 ()                  const;          // Compare to 0.

//      Service conversions.
//      Number does not change but its internal representation may change.
  cBigNumber& fit   ();                                 // Normalize.
  cBigNumber& tab   ();                                 // Table of shifts.
  cBigNumber& gc    ();                                 // Optimize memory.
  cBigNumber& pack  ();                                 // Truncate memory.
const cBigNumber& fit   () const { return ((cBigNumber*)this)->fit();   }
const cBigNumber& tab   () const { return ((cBigNumber*)this)->tab();   }
const cBigNumber& gc    () const { return ((cBigNumber*)this)->gc();    }
const cBigNumber& pack  () const { return ((cBigNumber*)this)->pack();  }

//      Unary operations with accumulation.
  cBigNumber& operator ++();                            // Prefix increment.
  cBigNumber& operator --();                            // Prefix decrement.
  cBigNumber& neg   ()  { return setneg  (*this); }     // Inversion of sign.
//cBigNumber& compl ()  { return setcompl(*this); }     // Bit-wise inversion.
  cBigNumber& mul2  ();                                 // Multiplication to 2.
  cBigNumber& div2  ();                                 // Division to 2.
  cBigNumber& pow2  ();                                 // Square.
  cBigNumber& sqrt  ();                                 // Square root.

//      Assign either copy of number or result of operation.
//      Operands are passed as parameters.
  cBigNumber& set         (const cBigNumber&);          // Copying.
  cBigNumber& set         (long);
  cBigNumber& setneg      (const cBigNumber&);          // Inversion of sign.
  cBigNumber& setneg      (long);
  cBigNumber& setcompl    (const cBigNumber&);          // Bit-wise inversion.
  cBigNumber& setcompl    (long);
  cBigNumber& setxor (const cBigNumber&, const cBigNumber&);    // Bit-wise
  cBigNumber& setxor (const cBigNumber&, long);                 // sum by
  cBigNumber& setxor (long, const cBigNumber&);                 // module 2.
  cBigNumber& setxor (long, long);
  cBigNumber& setand (const cBigNumber&, const cBigNumber&);    // Bit-wise
  cBigNumber& setand (const cBigNumber&, long);                 // conjunction.
  cBigNumber& setand (long, const cBigNumber&);
  cBigNumber& setand (long, long);
  cBigNumber& setor  (const cBigNumber&, const cBigNumber&);    // Bit-wise
  cBigNumber& setor  (const cBigNumber&, long);                 // disjunction.
  cBigNumber& setor  (long, const cBigNumber&);
  cBigNumber& setor  (long, long);
  cBigNumber& setadd (const cBigNumber&, const cBigNumber&);    // Addition.
  cBigNumber& setadd (const cBigNumber&, long);
  cBigNumber& setadd (long, const cBigNumber&);
  cBigNumber& setadd (long, long);
  cBigNumber& setsub (const cBigNumber&, const cBigNumber&);    // Subtraction.
  cBigNumber& setsub (const cBigNumber&, long);
  cBigNumber& setsub (long, const cBigNumber&);
  cBigNumber& setsub (long, long);
  cBigNumber& setmul (const cBigNumber&, const cBigNumber&);    // Multiplication.
  cBigNumber& setmul (const cBigNumber&, long);
  cBigNumber& setmul (long, const cBigNumber&);
  cBigNumber& setmul (long, long);
  cBigNumber& setdiv (const cBigNumber&, const cBigNumber&);    // Division.
  cBigNumber& setdiv (const cBigNumber&, long);
  cBigNumber& setdiv (long, const cBigNumber&);
  cBigNumber& setdiv (long, long);
  cBigNumber& setmod (const cBigNumber&, const cBigNumber&);    // Module.
  cBigNumber& setmod (const cBigNumber&, long);
  cBigNumber& setmod (long, const cBigNumber&);
  cBigNumber& setmod (long, long);
  cBigNumber& setshl (const cBigNumber&, const cBigNumber&);    // Left
  cBigNumber& setshl (const cBigNumber&, long);                 // shift.
  cBigNumber& setshl (long, const cBigNumber&);
  cBigNumber& setshl (long, long);
  cBigNumber& setshr (const cBigNumber&, const cBigNumber&);    // Right
  cBigNumber& setshr (const cBigNumber&, long);                 // shift.
  cBigNumber& setshr (long, const cBigNumber&);
  cBigNumber& setshr (long, long);
  cBigNumber& setpow (const cBigNumber&, const cBigNumber&);    // Power.
  cBigNumber& setpow (const cBigNumber&, long);
  cBigNumber& setpow (long, const cBigNumber&);
  cBigNumber& setpow (long, long);
  cBigNumber& setpowmod (const cBigNumber&, const cBigNumber&,  // Power
                         const cBigNumber&);                    // by module.
  cBigNumber& setpowmod (const cBigNumber&, long, const cBigNumber&);
  cBigNumber& setpowmod (long, const cBigNumber&, const cBigNumber&);
  cBigNumber& setpowmod (long, long, const cBigNumber&);
  cBigNumber& setpowmod (const cBigNumber&, const cBigNumber&, long);
  cBigNumber& setpowmod (const cBigNumber&, long, long);
  cBigNumber& setpowmod (long, const cBigNumber&, long);
  cBigNumber& setpowmod (long, long, long);
  cBigNumber& setsqrt   (const cBigNumber&);                    // Square root.
  cBigNumber& setsqrt   (long);

//      Assignment.
  cBigNumber& operator =  (const cBigNumber& b) { return set (b);       }
  cBigNumber& operator =  (long b)              { return set (b);       }

//      Binary operations with accumulation.
//      Second operand are passed as parameter.
  cBigNumber& operator ^= (const cBigNumber&);  // Bit-wise sum by mod 2
  cBigNumber& operator ^= (long);
  cBigNumber& operator &= (const cBigNumber&);  // Bit-wise conjunction.
  cBigNumber& operator &= (long);
  cBigNumber& operator |= (const cBigNumber&);  // Bit-wise disjunction.
  cBigNumber& operator |= (long);
  cBigNumber& operator += (const cBigNumber&);  // Addition.
  cBigNumber& operator += (long);
  cBigNumber& operator -= (const cBigNumber&);  // Subtraction.
  cBigNumber& operator -= (long);
  cBigNumber& operator *= (const cBigNumber&);  // Multiplication.
  cBigNumber& operator *= (long);
  cBigNumber& operator /= (const cBigNumber&);  // Division.
  cBigNumber& operator /= (long);
  cBigNumber& operator %= (const cBigNumber&);  // Module.
  cBigNumber& operator %= (long);
  cBigNumber& operator<<= (const cBigNumber&);  // Left shift.
  cBigNumber& operator<<= (long);
  cBigNumber& operator>>= (const cBigNumber&);  // Right shift.
  cBigNumber& operator>>= (long);
  cBigNumber& pow         (const cBigNumber&);  // Power.
  cBigNumber& pow         (long);
  cBigNumber& powmod      (const cBigNumber&, const cBigNumber&);
  cBigNumber& powmod      (const cBigNumber&, long);
  cBigNumber& powmod      (long, const cBigNumber&);
  cBigNumber& powmod      (long, long);

//      Combined operation of multiplication with accumulation.
//      Operands are passed as parameters, result of multiplication
//      is added to accumulator.
  cBigNumber& addmul      (const cBigNumber&, const cBigNumber&);
  cBigNumber& addmul      (const cBigNumber&, long);
  cBigNumber& addmul      (long, const cBigNumber&);
  cBigNumber& addmul      (long, long);

//      Combined operation of division with module.
//      Module is assigned to dividend, which must not overlap with result.
  cBigNumber& setdivmod   (cBigNumber&, const cBigNumber&);
  cBigNumber& setdivmod   (cBigNumber&, long);

//      Combined operation of square root with remainder.
//      Remainder is assigned to argument, which must not overlap with result.
  cBigNumber& setsqrtrm   (cBigNumber&);

//      Multiplication with accumulation of preliminary prepared operands.
//      Multiplicand must contain table of shifts.
//      Multiplier must be non-negative.
//      Operands must not overlap with buffer of result.
  cBigNumber& addmultab   (const cBigNumber&, const cBigNumber&, size_t k = 0);
  cBigNumber& addmultab   (const cBigNumber&, long, size_t k = 0);

//      Division of preliminary prepared operands.
//      Operands must have identical signs. Dividend must be normalized,
//      divider must contain table of shifts and must not overlap with
//      buffer of result.
  cBigNumber& setdivtab (const cBigNumber&, const cBigNumber&); // Division.
  cBigNumber& setdivtab (long, const cBigNumber&);
  cBigNumber& setmodtab (const cBigNumber&, const cBigNumber&); // Module.
  cBigNumber& setmodtab (long, const cBigNumber&);

//      Division with module of preliminary prepared operands.
//      Operands must have identical signs, must not overlap with buffer of
//      result and each other. Dividend must be normalized,
//      divider must contain table of shifts.
  cBigNumber& setdivmodtab    (cBigNumber&, const cBigNumber&); // Division
                                                                // with module.
//      String conversions.
  cBigNumber& set         (const char *psz,             // Conversion of
                           unsigned radix = 10);        // string to number.

  char*       toatmp      (unsigned radix = 10,         // Preparing of
                           ios& os = *(ios*)NULL        // string in the
                          )   const;                    // static buffer.

//      Conversion to integer with check of range.
  long        tolong      ()  const;
  int         toint       ()  const;
  short       toshort     ()  const;

//      Conversions for logical C operations.
  operator  const void *  ()  const     { return (comp0()? e: 0);       }
  int         operator !  ()  const     { return (comp0() == 0);        }

//      Control of division.
static  void        maskdiv0   (int);   // Mask division by 0.
static  int         testdiv0   ();      // Is where division by 0?

//      Internal static buffers.
static  cBigNumber& lastdivmod ();      // Module of last division.
static  cBigNumber& lastrootrm ();      // Remainder of last square root.

//      Auxiliary methods.
  cBigNumber& set_        (const EXPTR(char) ps,        // Conversion of char
                           unsigned radix,              // array to unsigned
                           unsigned len);               // unlimited number.

private:
//      Auxiliary private methods.
  cBigNumber& setpow_    ();
  cBigNumber& setpowmod_ (const cBigNumber&);
  cBigNumber& setpowmod_ (long);
};

//================================================
//      Conventional stream IO.
//================================================
istream& operator >> (istream&, cBigNumber&);
ostream& operator << (ostream&, const cBigNumber&);

//================================================
//      Basic tests for primality of numbers.
//================================================

//      Strong Probable Primality test for base b
int     b_SPRP (const cBigNumber& n, const cBigNumber& b);

//================================================
//      Integral tests for primality (prime.cpp).
//================================================

//      Miller Strong Probable Primality test
int     SPRP    (const cBigNumber& n);

//      Factor Primality Test
int     HasFactor (const cBigNumber& n, size_t start = 0, size_t stop = 0);

//      Combined (Factor + Miller) Primality Test
int     IsPrime (const cBigNumber& n);

//================================================
//      Auxiliary class for passing of long
//      number to service functions.
//================================================

class _cBigLong
{
  long e [2];           // Work array.

public:                 // Assignment to work array.

  _cBigLong         (long n = 0) { e [0] = 1; e [1] = n;        }

  _cBigLong& operator = (long n) { e [1] = n; return *this;     }

#ifdef  NCHECKPTR

//      Conventional pointer for addressing without check of bounds.

  long*       base()            { return (e);                   }

#else

//      Restricted pointer for addressing with check of bounds:
//      on passing of index outside of array bounds function
//      assigned to ::exalloc_status.range_handler is called.

  _cBigLong&  block()           { return (*this);               }
  exptr<long> begin()           { return exptr<long> (e, 2);    }
  operator    exptr<long>()     { return exptr<long> (e, 2);    }

#endif
};

#ifdef __cplusplus
extern "C" {
#endif

//================================================
//      Service variables.
//================================================

extern  size_t     cBigNumberMaskDiv0;      // Mask for division by 0.
extern  cBigNumber cBigNumberLastDivMod;    // Module of last division.
extern  cBigNumber cBigNumberLastRootRm;    // Module of last square root.

//================================================
//      Service functions (Cbignumf.cpp).
//================================================

void    cBigNumberDump    (const EXPTR(long) p1, ostream& stream = cout);
void    cBigNumberERange  (const EXPTR(long) p1);
int     cBigNumberIsFit   (const EXPTR(long) p1);
int     cBigNumberBits    (const EXPTR(long) p1);
int     cLongBits         (long);
size_t  cBigNumberExpWords(const EXPTR(long) p1);
int     cBigNumberComp    (const EXPTR(long) p1, const EXPTR(long) p2);
void    cBigNumberCopy    (const EXPTR(long) p1, EXPTR(long) p);
void    cBigNumberCompl   (const EXPTR(long) p1, EXPTR(long) p);
void    cBigNumberCopyShl (const EXPTR(long) p1, size_t k1, EXPTR(long) p);
void    cBigNumberCopyShr (const EXPTR(long) p1, size_t k1, EXPTR(long) p);
void    cBigNumberCutOut  (const EXPTR(long) p1, size_t k1, size_t n,
                                 EXPTR(long) p);
size_t _cBigNumberNeg     (const EXPTR(long) p1, EXPTR(long) p);
size_t _cBigNumberMInc          (EXPTR(long) p1);
size_t _cBigNumberMDec          (EXPTR(long) p1);
size_t _cBigNumberMMul2         (EXPTR(long) p1);
size_t _cBigNumberMDiv2         (EXPTR(long) p1);
size_t _cBigNumberMul2    (const EXPTR(long) p1, EXPTR(long) p);
size_t _cBigNumberDiv2    (const EXPTR(long) p1, EXPTR(long) p);
size_t _cBigNumberFit           (EXPTR(long) p1);
size_t _cBigNumberFitTo         (EXPTR(long) p1, size_t n);
void    cBigNumberTab           (EXPTR(long) p1);
size_t _cBigNumberXor     (const EXPTR(long) p1, const EXPTR(long) p2,
                                 EXPTR(long) p);
size_t _cBigNumberAnd     (const EXPTR(long) p1, const EXPTR(long) p2,
                                 EXPTR(long) p);
size_t _cBigNumberOr      (const EXPTR(long) p1, const EXPTR(long) p2,
                                 EXPTR(long) p);
size_t _cBigNumberAdd     (const EXPTR(long) p1, const EXPTR(long) p2,
                                 EXPTR(long) p);
size_t _cBigNumberSub     (const EXPTR(long) p1, const EXPTR(long) p2,
                                 EXPTR(long) p);
size_t _cBigNumberSubS    (const EXPTR(long) p1, const EXPTR(long) p2,
                                 EXPTR(long) p);
size_t _cBigNumberMAdd          (EXPTR(long) p1, const EXPTR(long) p2);
size_t _cBigNumberMSub          (EXPTR(long) p1, const EXPTR(long) p2);
size_t  cBigNumberSkipLow0(const EXPTR(long) p1);
void    cBigNumberMAddM         (EXPTR(long) p1, const EXPTR(long) p2,
                                 size_t k2);
size_t _cBigNumberMSubD         (EXPTR(long) p1, const EXPTR(long) p2,
                                 size_t k2);
size_t _cBigNumberMMul2M        (EXPTR(long) p1);
size_t _cBigNumberMDiv2D        (EXPTR(long) p1);
void    cBigNumberMShrM         (EXPTR(long) p1);
void    cBigNumberMShlD         (EXPTR(long) p1);
void    cBigNumberMAddMulShl    (EXPTR(long) p1, const EXPTR(long) p2,
                                 size_t k, EXPTR(long) p);
void    cBigNumberMAddMulTab
                          (const EXPTR(long) p1, const EXPTR(long) p2,
                                 size_t k, EXPTR(long) p);
void    cBigNumberMAddMul (const EXPTR(long) p1, const EXPTR(long) p2,
                                 EXPTR(long) p);
void    cBigNumberMul     (const EXPTR(long) p1, const EXPTR(long) p2,
                                 EXPTR(long) p);
void    cBigNumberDiv0    (void);
void    cBigNumberMModDivTab    (EXPTR(long) p1, const EXPTR(long) p2,
                                 EXPTR(long) p);
void    cBigNumberMModDiv       (EXPTR(long) p1, const EXPTR(long) p2,
                                 EXPTR(long) p);
void    cBigNumberMModTab       (EXPTR(long) p1, const EXPTR(long) p2);
void    cBigNumberMMod          (EXPTR(long) p1, const EXPTR(long) p2);
void    cBigNumberPow           (EXPTR(long) p1, const EXPTR(long) p2,
                                 EXPTR(long) p);
void    cBigNumberPowMod        (EXPTR(long) p1, const EXPTR(long) p2,
                                                 const EXPTR(long) mod,
                                 EXPTR(long) p);
void    cBigNumberMRmSqrt       (EXPTR(long) p1, EXPTR(long) p);
#ifdef __cplusplus
}
#endif

//================================================
//      Service inline functions.
//================================================

inline  size_t  cBigNumberNeg     (const EXPTR(long) p1, EXPTR(long) p)
{
  return (size_t)(*(p)=_cBigNumberNeg (p1, p));
}

inline  size_t  cBigNumberMInc          (EXPTR(long) p1)
{
  return (size_t)(*(p1)=_cBigNumberMInc (p1));
}

inline  size_t  cBigNumberMDec          (EXPTR(long) p1)
{
  return (size_t)(*(p1)=_cBigNumberMDec (p1));
}

inline  size_t  cBigNumberMMul2         (EXPTR(long) p1)
{
  return (size_t)(*(p1)=_cBigNumberMMul2 (p1));
}

inline  size_t  cBigNumberMDiv2         (EXPTR(long) p1)
{
  return (size_t)(*(p1)=_cBigNumberMDiv2 (p1));
}

inline  size_t  cBigNumberMul2   (const EXPTR(long) p1, EXPTR(long) p)
{
  return (size_t)(*(p)=_cBigNumberMul2 (p1, p));
}

inline  size_t  cBigNumberDiv2   (const EXPTR(long) p1, EXPTR(long) p)
{
  return (size_t)(*(p)=_cBigNumberDiv2 (p1, p));
}

inline  size_t  cBigNumberFit          (EXPTR(long) p1)
{
  return (size_t)(*(p1)=_cBigNumberFit (p1));
}

inline  size_t  cBigNumberFitTo        (EXPTR(long) p1, size_t n)
{
  return (size_t)(*(p1)=_cBigNumberFitTo (p1, n));
}

inline  size_t  cBigNumberXor    (const EXPTR(long) p1, const EXPTR(long) p2,
                                        EXPTR(long) p)
{
  return (size_t)(*(p)=_cBigNumberXor (p1, p2, p));
}

inline  size_t  cBigNumberAnd    (const EXPTR(long) p1, const EXPTR(long) p2,
                                        EXPTR(long) p)
{
  return (size_t)(*(p)=_cBigNumberAnd (p1, p2, p));
}

inline  size_t  cBigNumberOr     (const EXPTR(long) p1, const EXPTR(long) p2,
                                        EXPTR(long) p)
{
  return (size_t)(*(p)=_cBigNumberOr  (p1, p2, p));
}

inline  size_t  cBigNumberAdd    (const EXPTR(long) p1, const EXPTR(long) p2,
                                        EXPTR(long) p)
{
  return (size_t)(*(p)=_cBigNumberAdd (p1, p2, p));
}

inline  size_t  cBigNumberSub    (const EXPTR(long) p1, const EXPTR(long) p2,
                                        EXPTR(long) p)
{
  return (size_t)(*(p)=_cBigNumberSub (p1, p2, p));
}

inline  size_t  cBigNumberSubS   (const EXPTR(long) p1, const EXPTR(long) p2,
                                        EXPTR(long) p)
{
  return (size_t)(*(p)=_cBigNumberSubS (p1, p2, p));
}

inline  size_t  cBigNumberMAdd         (EXPTR(long) p1, const EXPTR(long) p2)
{
  return (size_t)(*(p1)=_cBigNumberMAdd (p1, p2));
}

inline  size_t  cBigNumberMSub         (EXPTR(long) p1, const EXPTR(long) p2)
{
  return (size_t)(*(p1)=_cBigNumberMSub (p1, p2));
}

inline  size_t  cBigNumberMSubD        (EXPTR(long) p1, const EXPTR(long) p2,
                                        size_t k2)
{
  return (size_t)(*(p1)=_cBigNumberMSubD (p1, p2, k2));
}

inline  size_t  cBigNumberMMul2M        (EXPTR(long) p1)
{
  return (size_t)(*(p1)=_cBigNumberMMul2M (p1));
}

inline  size_t  cBigNumberMDiv2D        (EXPTR(long) p1)
{
  return (size_t)(*(p1)=_cBigNumberMDiv2D (p1));
}

//================================================
//      Implementation of constructors.
//================================================

inline cBigNumber::cBigNumber (const cBigNumber& b)
{
  e = NULL; len = 0;                    // Initialization by 0,
  checkexpand (b.length());             // Set size.
  cBigNumberCopy (EXPTRTYPE(b), EXPTRTYPE(*this));      // Copying.
}

inline cBigNumber::cBigNumber (const char *psz, unsigned radix)
{
  create();                             // Create number.
  set (psz, radix);                     // Parse constant.
}

//================================================
//      Implementation of information methods.
//================================================

inline void cBigNumber::dump()      const
{
  cBigNumberDump (EXPTRTYPE(*this));
}

inline void cBigNumber::erange()    const
{
  cBigNumberERange (EXPTRTYPE(*this));
}

inline int  cBigNumber::isfit()     const
{
  return cBigNumberIsFit (EXPTRTYPE(*this));
}

inline int  cBigNumber::bits()      const
{
  return cBigNumberBits (EXPTRTYPE(*this));
}

//================================================
//      Implementation of comparison methods.
//================================================

inline int      cBigNumber::comp (const cBigNumber& b)  const
{
  return cBigNumberComp (EXPTRTYPE(*this), EXPTRTYPE(b));
}

inline int      cBigNumber::comp (long b)               const
{
  return cBigNumberComp (EXPTRTYPE(*this), EXPTRTYPE(_cBigLong(b)));
}

inline int      cBigNumber::comp0 ()                    const
{
  return cBigNumberComp (EXPTRTYPE(*this), EXPTRTYPE(_cBigLong(0)));
}

//================================================
//      Implementation of assign operations.
//================================================

inline cBigNumber& cBigNumber::set         (const cBigNumber& b)
{
  checkexpand (b.length()); cBigNumberCopy (EXPTRTYPE(b), EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

inline cBigNumber& cBigNumber::set         (long b)
{
  if (e == NULL) create();
  e[0] = 1; e[1] = b;
  checkindex (length());
  return *this;
}

inline cBigNumber& cBigNumber::setneg      (const cBigNumber& b)
{
  checkexpand (b.length() + 1); cBigNumberNeg (EXPTRTYPE(b), EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

inline cBigNumber& cBigNumber::setneg      (long b)
{
  checkexpand (2); e[0] = 1 + (b == LONG_MIN); e[1] = -b; e[2] = 0;
  checkindex (length());
  return *this;
}

inline cBigNumber& cBigNumber::setcompl    (const cBigNumber& b)
{
  checkexpand (b.length()); cBigNumberCompl (EXPTRTYPE(b), EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

inline cBigNumber& cBigNumber::setcompl    (long b)
{
  if (e == NULL) create();
  e[0] = 1; e[1] = ~b;
  checkindex (length());
  return *this;
}

inline cBigNumber& cBigNumber::operator ^= (const cBigNumber& b)
{
  return setxor (*this, b);
}

inline cBigNumber& cBigNumber::operator ^= (long b)
{
  return setxor (*this, b);
}

inline cBigNumber& cBigNumber::operator &= (const cBigNumber& b)
{
  return setand (*this, b);
}

inline cBigNumber& cBigNumber::operator &= (long b)
{
  return setand (*this, b);
}

inline cBigNumber& cBigNumber::operator |= (const cBigNumber& b)
{
  return setor (*this, b);
}

inline cBigNumber& cBigNumber::operator |= (long b)
{
  return setor (*this, b);
}

inline cBigNumber& cBigNumber::operator += (const cBigNumber& b)
{
  return setadd (*this, b);
}

inline cBigNumber& cBigNumber::operator += (long b)
{
  return setadd (*this, b);
}

inline cBigNumber& cBigNumber::operator -= (const cBigNumber& b)
{
  return setsub (*this, b);
}

inline cBigNumber& cBigNumber::operator -= (long b)
{
  return setsub (*this, b);
}

inline cBigNumber& cBigNumber::operator *= (const cBigNumber& b)
{
  return setmul (*this, b);
}

inline cBigNumber& cBigNumber::operator *= (long b)
{
  return setmul (*this, b);
}

inline cBigNumber& cBigNumber::operator /= (const cBigNumber& b)
{
  swap (cBigNumberLastDivMod);
  checkexpand (cBigNumberLastDivMod.length() + 2);
  cBigNumberMModDiv (EXPTRTYPE(cBigNumberLastDivMod), EXPTRTYPE(b),
                     EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

inline cBigNumber& cBigNumber::operator /= (long b)
{
  swap (cBigNumberLastDivMod);
  checkexpand (cBigNumberLastDivMod.length() + 2);
  cBigNumberMModDiv (EXPTRTYPE(cBigNumberLastDivMod), EXPTRTYPE(_cBigLong(b)),
                     EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

inline cBigNumber& cBigNumber::operator %= (const cBigNumber& b)
{
  cBigNumberMMod (EXPTRTYPE(*this), EXPTRTYPE(b));
  checkindex (length());
  return *this;
}

inline cBigNumber& cBigNumber::operator %= (long b)
{
  cBigNumberMMod (EXPTRTYPE(*this), EXPTRTYPE(_cBigLong(b)));
  checkindex (length());
  return *this;
}

inline cBigNumber& cBigNumber::pow (const cBigNumber& b)
{
  return setpow (*this, b);
}

inline cBigNumber& cBigNumber::pow (long b)
{
  return setpow (*this, b);
}

inline cBigNumber& cBigNumber::powmod (const cBigNumber& b,
                                       const cBigNumber& mod)
{
  return setpowmod (*this, b, mod);
}

inline cBigNumber& cBigNumber::powmod (const cBigNumber& b, long mod)
{
  return setpowmod (*this, b, mod);
}

inline cBigNumber& cBigNumber::powmod (long b, const cBigNumber& mod)
{
  return setpowmod (*this, b, mod);
}

inline cBigNumber& cBigNumber::powmod (long b, long mod)
{
  return setpowmod (*this, b, mod);
}

//================================================
//      Implementation of compare operations.
//================================================

inline  int  operator == (const cBigNumber& a, const cBigNumber& b)
{
  return (a.comp(b) == 0);
}

inline  int  operator == (const cBigNumber& a, long b)
{
  return (a.comp(b) == 0);
}

inline  int  operator == (long b, const cBigNumber& a)
{
  return (a.comp(b) == 0);
}

inline  int  operator != (const cBigNumber& a, const cBigNumber& b)
{
  return (a.comp(b) != 0);
}

inline  int  operator != (const cBigNumber& a, long b)
{
  return (a.comp(b) != 0);
}

inline  int  operator != (long b, const cBigNumber& a)
{
  return (a.comp(b) != 0);
}

inline  int  operator >= (const cBigNumber& a, const cBigNumber& b)
{
  return (a.comp(b) >= 0);
}

inline  int  operator >= (const cBigNumber& a, long b)
{
  return (a.comp(b) >= 0);
}

inline  int  operator >= (long b, const cBigNumber& a)
{
  return (a.comp(b) <= 0);
}

inline  int  operator <= (const cBigNumber& a, const cBigNumber& b)
{
  return (a.comp(b) <= 0);
}

inline  int  operator <= (const cBigNumber& a, long b)
{
  return (a.comp(b) <= 0);
}

inline  int  operator <= (long b, const cBigNumber& a)
{
  return (a.comp(b) >= 0);
}

inline  int  operator >  (const cBigNumber& a, const cBigNumber& b)
{
  return (a.comp(b) > 0);
}

inline  int  operator >  (const cBigNumber& a, long b)
{
  return (a.comp(b) > 0);
}

inline  int  operator >  (long b, const cBigNumber& a)
{
  return (a.comp(b) < 0);
}

inline  int  operator <  (const cBigNumber& a, const cBigNumber& b)
{
  return (a.comp(b) < 0);
}

inline  int  operator <  (const cBigNumber& a, long b)
{
  return (a.comp(b) < 0);
}

inline  int  operator <  (long b, const cBigNumber& a)
{
  return (a.comp(b) > 0);
}

//================================================
//      Implementation of check of range.
//================================================

inline  long    cBigNumber::tolong()  const
{
  if (size() == 0)      return (0);
  if (length() > 1)     erange();
  return (hiword());
}

inline  int     cBigNumber::toint()   const
{
  long n = tolong();
  if ((int) n != n)     erange();
  return ((int)n);
}

inline  short   cBigNumber::toshort() const
{
  long n = tolong();
  if ((short) n != n)   erange();
  return ((short)n);
}

//================================================
//      Implementation of control methods.
//================================================

inline void     cBigNumber::maskdiv0 (int mask)
{
  cBigNumberMaskDiv0 = (mask << 1);
}

inline int      cBigNumber::testdiv0()
{
  return (cBigNumberMaskDiv0 == 1);
}

inline cBigNumber& cBigNumber::lastdivmod()
{
  return (cBigNumberLastDivMod);
}

inline cBigNumber& cBigNumber::lastrootrm()
{
  return (cBigNumberLastRootRm);
}

//================================================
//      Inherited classes for implementation
//      of operation which require for creation
//      of temporary objects.
//================================================

class cBigTemp: public cBigNumber               // Number in temporary object.
{
//      Class cBigTemp describes temporary objects
//      for results of arithmetical operations.
//
//      For optimization of memory allocation constructor cBigTemp.
//      uses stack of dynamic arrays, which allows to avoid redundant
//      allocations of memory: when object cBigTemp is deleted, pointer
//      to its base array is put to stack; when object cBigTemp is
//      created it receives base array from stack. Memory allocation
//      is required only when the stack if empty.
//
//      Constructors are protected to prevent explicit creation
//      of objects cBigTemp.

protected:

  static exblockstack<long> stack;      // Stack of dynamic arrays.

  cBigTemp (): cBigNumber (stack)   {}  // Constructor.
  ~cBigTemp()  { stack.put (*this);  }  // Destructor.
/*
  cBigTemp ()  {}                       // Constructor.
  ~cBigTemp()  {}                       // Destructor.
*/
//      Blocking of assignments to objects of class cBigNumber to prevent
//      incorrect assignments to temporary objects, such as (-x) = 10;

private:

  cBigNumber& neg   ();                         // Negation.
//cBigNumber& compl ();                         // Bit-wise inversion.
  cBigNumber& operator ++ ();                   // Prefix increment.
  cBigNumber& operator -- ();                   // Prefix decrement.
  cBigNumber& mul2  ();                         // Multiplication to 2.
  cBigNumber& div2  ();                         // Division to 2.
  cBigNumber& pow2  ();                         // Square.
  cBigNumber& fit   ();                         // Normalization.
  cBigNumber& tab   ();                         // Build table of shifts.
  cBigNumber& gc    ();                         // Optimization of memory.
  cBigNumber& pack  ();                         // Truncation of memory.
  cBigNumber& operator =  (const cBigNumber&);  // Bit-wise sum by mod 2
  cBigNumber& operator =  (long);
  cBigNumber& operator ^= (const cBigNumber&);  // Bit-wise sum by mod 2
  cBigNumber& operator ^= (long);
  cBigNumber& operator &= (const cBigNumber&);  // Bit-wise conjunction.
  cBigNumber& operator &= (long);
  cBigNumber& operator |= (const cBigNumber&);  // Bit-wise disjunction.
  cBigNumber& operator |= (long);
  cBigNumber& operator += (const cBigNumber&);  // Addition.
  cBigNumber& operator += (long);
  cBigNumber& operator -= (const cBigNumber&);  // Subtraction.
  cBigNumber& operator -= (long);
  cBigNumber& operator *= (const cBigNumber&);  // Multiplication.
  cBigNumber& operator *= (long);
  cBigNumber& operator /= (const cBigNumber&);  // Division.
  cBigNumber& operator /= (long);
  cBigNumber& operator %= (const cBigNumber&);  // Module.
  cBigNumber& operator %= (long);
  cBigNumber& operator<<= (const cBigNumber&);  // Left shift.
  cBigNumber& operator<<= (long);
  cBigNumber& operator>>= (const cBigNumber&);  // Right shift.
  cBigNumber& operator>>= (long);
  cBigNumber& pow         (const cBigNumber&);  // Power.
  cBigNumber& pow         (long);
  cBigNumber& addmul      (const cBigNumber&, const cBigNumber&);
  cBigNumber& addmul      (const cBigNumber&, long);
  cBigNumber& addmul      (long, const cBigNumber&);
  cBigNumber& addmul      (long, long);
};

struct cBigInc: public cBigTemp                 // Postfix increment.
{
  cBigInc   (cBigNumber& a)                             { set (a); ++a; }
};

struct cBigDec: public cBigTemp                 // Postfix decrement.
{
  cBigDec   (cBigNumber& a)                             { set (a); --a; }
};

struct cBigPos: public cBigTemp                 // Unary +.
{
  cBigPos   (const cBigNumber& a)                       { set (a);      }
  cBigPos   (long a)                                    { set (a);      }
};

struct cBigNeg: public cBigTemp                 // Unary -.
{
  cBigNeg   (const cBigNumber& a)                       { setneg (a);   }
  cBigNeg   (long a)                                    { setneg (a);   }
};

struct cBigCompl: public cBigTemp               // Bit-wise inversion.
{
  cBigCompl (const cBigNumber& a)                       { setcompl (a); }
  cBigCompl (long a)                                    { setcompl (a); }
};

struct cBigXor: public cBigTemp                 // Bit-wise sum by module 2.
{
  cBigXor (const cBigNumber& a, const cBigNumber& b)    { setxor (a, b); }
  cBigXor (const cBigNumber& a, long b)                 { setxor (a, b); }
  cBigXor (long a, const cBigNumber& b)                 { setxor (a, b); }
  cBigXor (long a, long b)                              { setxor (a, b); }
};

struct cBigAnd: public cBigTemp                 // Bit-wise conjunction.
{
  cBigAnd (const cBigNumber& a, const cBigNumber& b)    { setand (a, b); }
  cBigAnd (const cBigNumber& a, long b)                 { setand (a, b); }
  cBigAnd (long a, const cBigNumber& b)                 { setand (a, b); }
  cBigAnd (long a, long b)                              { setand (a, b); }
};

struct cBigOr : public cBigTemp                 // Bit-wise disjunction.
{
  cBigOr  (const cBigNumber& a, const cBigNumber& b)    { setor  (a, b); }
  cBigOr  (const cBigNumber& a, long b)                 { setor  (a, b); }
  cBigOr  (long a, const cBigNumber& b)                 { setor  (a, b); }
  cBigOr  (long a, long b)                              { setor  (a, b); }
};

struct cBigAdd: public cBigTemp                         // Addition.
{
  cBigAdd (const cBigNumber& a, const cBigNumber& b)    { setadd (a, b); }
  cBigAdd (const cBigNumber& a, long b)                 { setadd (a, b); }
  cBigAdd (long a, const cBigNumber& b)                 { setadd (a, b); }
  cBigAdd (long a, long b)                              { setadd (a, b); }
};

struct cBigSub: public cBigTemp                         // Subtraction.
{
  cBigSub (const cBigNumber& a, const cBigNumber& b)    { setsub (a, b); }
  cBigSub (const cBigNumber& a, long b)                 { setsub (a, b); }
  cBigSub (long a, const cBigNumber& b)                 { setsub (a, b); }
  cBigSub (long a, long b)                              { setsub (a, b); }
};

struct cBigMul: public cBigTemp                         // Multiplication.
{
  cBigMul (const cBigNumber& a, const cBigNumber& b)    { setmul (a, b); }
  cBigMul (const cBigNumber& a, long b)                 { setmul (a, b); }
  cBigMul (long a, const cBigNumber& b)                 { setmul (a, b); }
  cBigMul (long a, long b)                              { setmul (a, b); }
};

struct cBigDiv: public cBigTemp                         // Division.
{
  cBigDiv (const cBigNumber& a, const cBigNumber& b)    { setdiv (a, b); }
  cBigDiv (const cBigNumber& a, long b)                 { setdiv (a, b); }
  cBigDiv (long a, const cBigNumber& b)                 { setdiv (a, b); }
  cBigDiv (long a, long b)                              { setdiv (a, b); }
};

struct cBigMod: public cBigTemp                         // Module.
{
  cBigMod (const cBigNumber& a, const cBigNumber& b)    { setmod (a, b); }
  cBigMod (const cBigNumber& a, long b)                 { setmod (a, b); }
  cBigMod (long a, const cBigNumber& b)                 { setmod (a, b); }
  cBigMod (long a, long b)                              { setmod (a, b); }
};

struct cBigShl: public cBigTemp                         // Left shift.
{
  cBigShl (const cBigNumber& a, const cBigNumber& b)    { setshl (a, b); }
  cBigShl (const cBigNumber& a, long b)                 { setshl (a, b); }
  cBigShl (long a, const cBigNumber& b)                 { setshl (a, b); }
  cBigShl (long a, long b)                              { setshl (a, b); }
};

struct cBigShr: public cBigTemp                         // Right shift.
{
  cBigShr (const cBigNumber& a, const cBigNumber& b)    { setshr (a, b); }
  cBigShr (const cBigNumber& a, long b)                 { setshr (a, b); }
  cBigShr (long a, const cBigNumber& b)                 { setshr (a, b); }
  cBigShr (long a, long b)                              { setshr (a, b); }
};

struct cBigPow: public cBigTemp                         // Power.
{
  cBigPow (const cBigNumber& a, const cBigNumber& b)    { setpow (a, b); }
  cBigPow (const cBigNumber& a, long b)                 { setpow (a, b); }
  cBigPow (long a, const cBigNumber& b)                 { setpow (a, b); }
  cBigPow (long a, long b)                              { setpow (a, b); }
};

struct cBigPowMod: public cBigTemp                      // Power by module.
{
  cBigPowMod (const cBigNumber& a, const cBigNumber& b, cBigNumber& mod)
                                                { setpowmod (a, b, mod); }
  cBigPowMod (const cBigNumber& a, long b, cBigNumber& mod)
                                                { setpowmod (a, b, mod); }
  cBigPowMod (long a, const cBigNumber& b, cBigNumber& mod)
                                                { setpowmod (a, b, mod); }
  cBigPowMod (long a, long b, cBigNumber& mod)
                                                { setpowmod (a, b, mod); }
  cBigPowMod (const cBigNumber& a, const cBigNumber& b, long mod)
                                                { setpowmod (a, b, mod); }
  cBigPowMod (const cBigNumber& a, long b, long mod)
                                                { setpowmod (a, b, mod); }
  cBigPowMod (long a, const cBigNumber& b, long mod)
                                                { setpowmod (a, b, mod); }
  cBigPowMod (long a, long b, long mod)
                                                { setpowmod (a, b, mod); }
};

struct cBigSqrt: public cBigTemp                        // Square root.
{
  cBigSqrt (const cBigNumber& a)                        { setsqrt (a);   }
  cBigSqrt (long a)                                     { setsqrt (a);   }
};

//================================================
//      Implementation of unary operations, which
//      require for creation of temporary object.
//================================================

inline cBigInc  operator ++ (cBigNumber &a, int)
{
  return cBigInc (a);
}

inline cBigDec  operator -- (cBigNumber &a, int)
{
  return cBigDec (a);
}

inline cBigPos   operator + (const cBigNumber &a)
{
  return cBigPos (a);
}

inline cBigNeg   operator - (const cBigNumber &a)
{
  return cBigNeg (a);
}

inline cBigCompl operator ~ (const cBigNumber &a)
{
  return cBigCompl (a);
}

//================================================
//      Implementation of binary operations, which
//      require for creation of temporary object.
//================================================

inline cBigXor operator ^ (const cBigNumber& a, const cBigNumber& b)
{
  return cBigXor (a, b);
}

inline cBigXor operator ^ (const cBigNumber& a, long b)
{
  return cBigXor (a, b);
}

inline cBigXor operator ^ (long b, const cBigNumber& a)
{
  return cBigXor (a, b);
}

inline cBigAnd operator & (const cBigNumber& a, const cBigNumber& b)
{
  return cBigAnd (a, b);
}

inline cBigAnd operator & (const cBigNumber& a, long b)
{
  return cBigAnd (a, b);
}

inline cBigAnd operator & (long b, const cBigNumber& a)
{
  return cBigAnd (a, b);
}

inline cBigOr  operator | (const cBigNumber& a, const cBigNumber& b)
{
  return cBigOr  (a, b);
}

inline cBigOr  operator | (const cBigNumber& a, long b)
{
  return cBigOr  (a, b);
}

inline cBigOr  operator | (long b, const cBigNumber& a)
{
  return cBigOr  (a, b);
}

inline cBigAdd operator + (const cBigNumber& a, const cBigNumber& b)
{
  return cBigAdd (a, b);
}

inline cBigAdd operator + (const cBigNumber& a, long b)
{
  return cBigAdd (a, b);
}

inline cBigAdd operator + (long b, const cBigNumber& a)
{
  return cBigAdd (a, b);
}

inline cBigSub operator - (const cBigNumber& a, const cBigNumber& b)
{
  return cBigSub (a, b);
}

inline cBigSub operator - (const cBigNumber& a, long b)
{
  return cBigSub (a, b);
}

inline cBigSub operator - (long a, const cBigNumber& b)
{
  return cBigSub (a, b);
}

inline cBigMul operator * (const cBigNumber& a, const cBigNumber& b)
{
  return cBigMul (a, b);
}

inline cBigMul operator * (const cBigNumber& a, long b)
{
  return cBigMul (a, b);
}

inline cBigMul operator * (long b, const cBigNumber& a)
{
  return cBigMul (a, b);
}

inline cBigDiv operator / (const cBigNumber& a, const cBigNumber& b)
{
  return cBigDiv (a, b);
}

inline cBigDiv operator / (const cBigNumber& a, long b)
{
  return cBigDiv (a, b);
}

inline cBigDiv operator / (long a, const cBigNumber& b)
{
  return cBigDiv (a, b);
}

inline cBigMod operator % (const cBigNumber& a, const cBigNumber& b)
{
  return cBigMod (a, b);
}

inline cBigMod operator % (const cBigNumber& a, long b)
{
  return cBigMod (a, b);
}

inline cBigMod operator % (long a, const cBigNumber& b)
{
  return cBigMod (a, b);
}

inline cBigShl operator << (const cBigNumber& a, const cBigNumber& b)
{
  return cBigShl (a, b);
}

inline cBigShl operator << (const cBigNumber& a, long b)
{
  return cBigShl (a, b);
}

inline cBigShl operator << (long a, const cBigNumber& b)
{
  return cBigShl (a, b);
}

inline cBigShr operator >> (const cBigNumber& a, const cBigNumber& b)
{
  return cBigShr (a, b);
}

inline cBigShr operator >> (const cBigNumber& a, long b)
{
  return cBigShr (a, b);
}

inline cBigShr operator >> (long a, const cBigNumber& b)
{
  return cBigShr (a, b);
}

#endif//_CBIGNUM_H
