/* --------------------------------------------------------------
    Signed integers with unlimited range (version 1.1a).
    Input-output.

    http://www.imach.uran.ru/cbignum

    Copyright 1999-2005 by Raul N.Shakirov, IMach of RAS(UB).
    All Rights Reserved.

    Permission has been granted to copy, distribute and modify
    software in any context without fee, including a commercial
    application, provided that the aforesaid copyright statement
    is present here as well as exhaustive description of changes.

    THE SOFTWARE IS DISTRIBUTED "AS IS". NO WARRANTY OF ANY KIND
    IS EXPRESSED OR IMPLIED. YOU USE AT YOUR OWN RISK. THE AUTHOR
    WILL NOT BE LIABLE FOR DATA LOSS, DAMAGES, LOSS OF PROFITS OR
    ANY OTHER KIND OF LOSS WHILE USING OR MISUSING THIS SOFTWARE.
-------------------------------------------------------------- */
#ifdef  _MSC_VER
#pragma warning(disable:4995)
#endif/*_MSC_VER*/
#if __TURBOC__ > 0x410
#pragma option -xd-
#endif
#include <assert.h>
#include <ctype.h>
#include <stdlib.h>
#include <iomanip.h>
#include <limits.h>
#include "Cbignum.h"

//================================================
//      Tables.
//================================================

//      Maximal radix.

#define MAX_RADIX 16

//      Minimal size of number in KBytes to turn on progress
//      indicator on input/output.

#define MIN_PROGR 10

//      Table for conversion of digits to symbols.

static const char numtochar [2] [MAX_RADIX + 1] =
                               {{'0','1','2','3','4','5','6','7','8','9',
                                 'a','b','c','d','e','f','x'},
                                {'0','1','2','3','4','5','6','7','8','9',
                                 'A','B','C','D','E','F','X'}};

//      Table for conversion of symbols to digits (build by cBigNumberSqrTab)

static char chartonum [UCHAR_MAX + 1] = { 0 };

//      Table of coefficients contains maximal integer power of type long
//      and number of symbols in string representation of the aforesaid
//      power for each radix.

static const long maxradix [] [4] = {{ 0L,  1},     //  0
                                     { 1L,  1},     //  1
                            { 1073741824L, 30},     //  2
                            { 1162261467L, 19},     //  3
                            { 1073741824L, 15},     //  4
                            { 1220703125L, 13},     //  5
                            {  362797056L, 11},     //  6
                            { 1977326743L, 11},     //  7
                            { 1073741824L, 10},     //  8
                            {  387420489L,  9},     //  9
                            { 1000000000L,  9},     // 10
                            {  214358881L,  8},     // 11
                            {  429981696L,  8},     // 12
                            {  815730721L,  8},     // 13
                            { 1475789056L,  8},     // 14
                            {  170859375L,  7},     // 15
                            {  268435456L,  7} };   // 16

//      For optimization of input/output of unlimited numbers for
//      each radix array with results co consequent squaring of
//      coefficient is created. For each coefficient table of shifts
//      is generated. Maximal coefficient MAX_NP is determined by
//      the criteria of minimization of input/output time for numbers
//      of size 1,000,000 bit on processor with fast cache of size >=256K.

#define MAX_NP_IN  ((sizeof(size_t))==2? 2: 9)
#define MAX_NP_OUT ((sizeof(size_t))==2? 2: 9)
#define MAX_NP    (MAX_NP_IN>MAX_NP_OUT?MAX_NP_IN:MAX_NP_OUT)

static exarray<cBigNumber> powradix [MAX_RADIX + 1];

//      Filling of conversion table and array of coefficients.
//      Table must contain low coefficient and its square.

static  size_t  cBigNumberSqrTab (
                unsigned radix,         // Radix (2..16).
                size_t len,             // Number of words in number.
                size_t max_np           // Maximal coefficient.
)                                       // Returns required number of
{                                       // coefficients (0..MAX_NP).
  assert (radix >= 2);
  assert (radix <= MAX_RADIX);
  assert (max_np <= MAX_NP);
  static size_t npow [MAX_RADIX + 1];   // Number of coefficients.

  if (npow [radix] == 0)
  {
//      Conversion table.

    int i;
    for (i = 0; i <= UCHAR_MAX; i++)
    {
      chartonum [i] = MAX_RADIX;
    }
    for (i = 0; i < MAX_RADIX; i++)
    {
      chartonum [numtochar [0] [i]] = (char)i;
      chartonum [numtochar [1] [i]] = (char)i;
    }

//      The lowest coefficient and his square.

    powradix [radix] [0] = maxradix [radix] [0];
    powradix [radix] [0].tab();         // Table of shifts.
    powradix [radix] [1] = maxradix [radix] [0];
    powradix [radix] [1].pow2();        // Square of coefficient.
    powradix [radix] [1].tab();         // Table of shifts.
    npow [radix] = 2;                   // Count of coefficients.
  }

//      Higher coefficients, if required.

  size_t np = 0;                        // Index of coefficient.
  while (np < max_np && powradix [radix] [np].length() * 4 < len)
  {
    if (++np >= npow [radix])
    {
      powradix [radix] [np] = powradix [radix] [np - 1];
      powradix [radix] [np].pow2();     // Square of coefficient.
      powradix [radix] [np].tab();      // Table of shifts.
      ++ npow [radix];                  // Count of coefficients.
    }
  }
  assert (np <= max_np);
//cerr << np << endl;
  return np;
}

//      Work buffers.

static cBigNumber           num_;           // Long number.
static cBigNumber           mod_;           // Module of division.
static exvector<cBigNumber> stk_(MAX_NP);   // Stack of intermediate numbers.
static exvector<int>       kstk_(MAX_NP);   // Stack of flags.

//================================================
//      Conversion of string to unlimited number.
//================================================

//      Conversion of char array to unsigned unlimited number of radix 2..16.

cBigNumber& cBigNumber::set_ (          // Conversion from char array.
                const EXPTR(char) ps,   // Char array.
                unsigned radix,         // Radix (2..16).
                unsigned len            // Length of char array.
            )
{
  assert (ps);
  assert (radix >= 2);
  assert (radix <= MAX_RADIX);

//      Input of number.
//
//      Initial (non-optimized algorithm looks as the following:
//      *this = 0;
//      for (; len > 0; --len)
//      {
//        *this *= radix;
//        *this += chartonum [*ps++];
//      }
//
//      In optimized algorithm we use method addmultab for
//      multiplication with accumulation. To reduce number of
//      long multiplications we use multiplication to coefficients
//      powradix, obtained by consequent squaring of radix.
//      Results of consequent multiplications with accumulation
//      are stored in num_ and stack of multipliers mul.
//      Initially stack is empty.
//
//      ALGORITHM
//      Multipliers of 0-th order are represented by long word
//      and are obtained by conversion of short subchains.
//      Multiplier of i-th order is obtained by addmultab as
//      accumulation of two consequent multipliers of order i-1,
//      first of which is stored in [i].
//      Result number is obtained by addmultab as consequent
//      accumulation of highest multiplier stored in num_.

  *this = 0;
  if (len != 0)
  {
    long lradix =       maxradix [radix] [0];
    int  nradix = (int) maxradix [radix] [1];
    int nr = (len - 1) / nradix;        // Number of full words (no more).
    int n = len - nr++ * nradix;        // Number of symbols in last word.

//      Filling of stack of coefficients powradix [radix].
//      Required number of coefficients is returned.

    size_t np = cBigNumberSqrTab (radix, nr, MAX_NP_IN);
    EXPTR(cBigNumber) powr = EXPTRTYPE(powradix [radix]);

//      Zero-filling of stack of multipliers.

    EXPTR(cBigNumber)  mul = EXPTRTYPE(stk_);   // Stack of multipliers.
    { for (size_t i = 1; i < np; i++) mul [i] = 0; }

//      Progress indication.

    int numlength = nr / (1024 / sizeof (long));
    if (numlength < MIN_PROGR) numlength = 0;   // Size if KBytes.
    if (numlength > 0)
    {
      cerr << "Converting"
           << setw (7) << 0         <<"K of"
           << setw (7) << numlength <<"K\b"
           << "\b\b\b\b\b\b\b\b\b\b\b\b\b\b"
           << "\b\b\b\b\b\b\b\b\b\b\b\b\b\b";
    }

//      Cycle on pair groups of nradix digits.

    long lhigh = 0;                     // The high word.
    if ((nr & 1) != 0) goto odd;        // The lower word (may be incomplete).
    do
    {
//      Conversion of high group of nradix digits.

      {
        long lb = 0;                    // Word.
        for(;;)
        {
          lb += chartonum [*ps++];
          assert (lb >= 0);
          if (--n == 0) break;
          lb *= radix;
        }
        n = nradix;
        lhigh = lb;                     // The high word.
      }
      --nr;                             // Have converted the group.

//      Conversion of lower group of nradix digits.

odd:  {
        long lb = 0;                    // Word.
        for(;;)
        {
          lb += chartonum [*ps++];
          assert (lb >= 0);
          if (--n == 0) break;
          lb *= radix;
        }
        n = nradix;
        num_ = lb;                      // Lower word.
      }
      --nr;                             // Have converted the group.

//      Calculation of multiplier of 1st order.

      num_.addmul (lhigh, lradix);      // Accumulation of words.
      assert (num_ >= 0);

//      Calculation of multipliers of 2nd and higher order.
//      and accumulation of the number.

      for (size_t i = 1;; i++)
      {
        if (i < np)                     // Calculation of multipliers.
        {
           if ((nr & (2<<i)-1) != 0)    // Odd multiplier
           {                            // is stored
             num_.swap (mul [i]);       // in the stack.
             break;
           }                            // Otherwise multiplier of
                                        // higher order is calculated.
           num_.addmultab (powr [i], mul [i]);
           assert (num_ >= 0);
        }
        else                            // Accumulation of the number.
        {
          if (numlength > 0)            // Progress indication.
          {
            cerr << "Converting"
                 << setw (7) << (numlength -
                                 nr / (1024 / sizeof (long)))
                                           <<"K of"
                 << setw (7) <<  numlength <<"K\b"
                 << "\b\b\b\b\b\b\b\b\b\b\b\b\b\b"
                 << "\b\b\b\b\b\b\b\b\b\b\b\b\b\b";
          }
          num_.addmultab (powr [i], *this);
          assert (num_ >= 0);
          num_.swap (*this);
          break;
        }
      }
    }
    while (nr != 0);

    if (numlength != 0)                 // Clearing of indicator.
          cerr << "                             \b"
               << "\b\b\b\b\b\b\b\b\b\b\b\b\b\b"
               << "\b\b\b\b\b\b\b\b\b\b\b\b\b\b";
  }

//      Conversion finished.

  EXDEBUG (num_ = 0);                   // Debug clearing.
  return *this;
}

//      Conversion of string to signed unlimited number of radix 2..16.
//      If radix is not indicated (set to 0) then radix is set to
//      10, 8 or 16 depending of prefixes 0, 0x, 0X.

cBigNumber& cBigNumber::set (           // Conversion from string.
                const char *psz,        // String.
                unsigned radix          // Radix (2..16) or 0.
            )                           // 0 - by C conventions.
{
  assert (psz);
  assert (radix != 1);
  assert (radix <= MAX_RADIX);
  int sign = 0;                         // Sign of number.
  unsigned c = *psz++;                  // Current symbol.

//      Recognition of sign.

  switch (c)
  {
    case '-': sign = 1;
    case '+': c = *psz++;
  }

//      Recognition of radix.

  if (radix == 0)
  {
    radix = 10;
    if (c == '0')
    {
      c = *psz++; radix = 8;
      if (c == 'X' || c == 'x') { c = *psz++; radix = 16; }
    }
  }

//      Find end of number.

  cBigNumberSqrTab (radix, 0, 0);       // Filling of chartonum.
  int len = 0;
  for (;;)
  {
    if ((size_t)chartonum [c] >= radix) break;
    c = psz [len++];                    // Next symbol.
  }
  --psz;

//      Decoding of number.

  set_ (EXPTRTO(char,psz,len), radix, len);

//      Assigning of sign.

  if (sign) this->neg();

  return *this;
}

//================================================
//      Operator of input.
//================================================

//      Formatting is controlled by the following flags:
//      ios::skipws     skip initial spaces
//      ios::dec        number of radix 10
//      ios::oct        number of radix 8
//      ios::hex        number of radix 16

istream& operator >> (istream& is, cBigNumber& b)
{
  assert (&is);
  assert (&b);
  static exarray<char> str_;            // Static string.
  int sign = 0;                         // Sign of number.
  char c;                               // Current symbol.
  long flags = is.flags();              // Flags.
  unsigned radix = (                    // Radix.
         (flags & ios::dec)? 10:
         (flags & ios::oct)? 8:
         (flags & ios::hex)? 16: 0);

//      Input of 1st symbol with skipping of spaces if ios::skipws is set.

  is >> c;

//      Recognition of sign.

  switch (c)
  {
    case '-': sign = 1;
    case '+': c = (char) is.get();
  }
  if (radix == 0)                       // Recognition of radix.
  {
    radix = 10;
    if (c == '0')
    {
      c = (char) is.get(); radix = 8;
      if (c == 'X' || c == 'x') { c = (char) is.get(); radix = 16; }
    }
  }

//      Input of symbols.

  cBigNumberSqrTab (radix, 0, 0);       // Filling of chartonum.
  int len = 0;
  for (;;)
  {
    if ((size_t)chartonum [(unsigned char)c] >= radix) break;
    str_ [len++] = c;
    c = (char) is.get();                // Next symbol.
  }
  if ((signed char)c != EOF)
  {
    is.putback (c);                     // Return non-recognized symbol.
  }
  str_ [len] = 0;                       // End of string.
  str_.checkshrink (len);

//      Decoding of number.

  b.set_ (EXPTRTYPE(str_), radix, len);

//      Assigning of sign.

  if (sign) b.neg();

  return is;
}

//================================================
//      Conversion of unlimited number to string.
//================================================

//      Conversion of unlimited number to string works for radixes 2..16
//      and accepts formatting flags of standard class ios.
//      ios::internal   sign and radix at left, number at right, filling
//                      by symbol ios::fill() up to ios::width() symbols.
//      ios::dec        signed number of radix 10 (default).
//      ios::oct        unsigned number of radix 8.
//      ios::hex        unsigned number of radix 16.
//      ios::showbase   add indicator of radix (0 or 0x).
//      ios::showpos    show sign for positive numbers.
//      ios::uppercase  upper-case letters for number with radix > 10.
//      If radix is not indicated (set to 0) then radix is determined
//      by flags ios:dec, ios::oct, ios::hex, otherwise number is
//      converted to signed form ignoring flags ios:dec, ios::oct, ios::hex.
//
//      Method returns pointer to static string which is
//      overwritten after each consequent call of method.

char*   cBigNumber::toatmp (            // Conversion to string.
                unsigned radix,         // Radix (2..16) or 0.
                ios& os                 // Formatting of NULL.
        ) const                         // Returns pointer to string
{                                       // in internal static buffer.
  assert (radix != 1);
  assert (radix <= MAX_RADIX);
  num_ = *this; num_.fit();             // Normalization.
  size_t i = 0;                         // Index in str_.
  long flags = (&os? os.flags(): 0);    // Formatting flags  ios.
  static exarray<char> str_;            // Static buffer.

//      Select conversion table.

  register const EXPTR(char) ps = EXPTRTO (char,
                         numtochar [(flags & ios::uppercase) != 0],
                         sizeof (numtochar [0]));

//      Output os sign and prefixes.

  {
    long sign = num_.comp0();

    if (radix == 0)                             // If no radix is indicated
    {                                           // determine its by flags.
      radix = ((flags & ios::hex)? 16: (flags & ios::oct)? 8: 10);
      if (radix != 10)                          // If radix is 8 or 16,
      {                                         // do not output sign.
        if (sign < 0)
        {
          ++(num_.e[0]);                        // Add 0-word
          num_.checkexpand (num_.length());     // to make the
          num_.e [num_.length()] = 0;           // number positive.
        }
        sign = 0;
      }
    }

    if (sign < 0)                                   // Output sign '-'.
    {
      str_ [i++] = '-'; num_.neg();
    }
    else if (sign > 0 && (flags & ios::showpos))    // Output sign '+'
    {
      str_ [i++] = '+';
    }

    if (radix % 8 == 0 && (flags & ios::showbase))  // Output radix.
    {
      str_ [i++] = '0'; if (radix == 16) str_ [i++] = ps [16];
    }
  }

//      Output of number in reverse order.
//
//      Initial (non-optimized algorithm looks as the following:
//      do
//      {
//        str_ [i++] = ps [num % radix];
//        num_ /= radix;
//      }
//      while (num_ != 0);
//
//      In optimized algorithm we use method setdivmodtab for
//      division with module. To reduce number of long divisions
//      we use division to coefficients powradix, obtained by
//      consequent squaring of radix.
//
//      As a result of division to the highest coefficient with index
//      npmax we obtain quotient and module, which are divided to
//      coefficient with index npmax-1, e.t.c, until as result
//      of division to lowest coefficient witc index 0 we obtain
//      number with fit to digital grid of type long. Long word
//      is converted to string be consequent division to radix.
//
//      The highest quotient is stored in num_. Lower quotients are stored
//      in the stack of quotients div and marked in the stack of flags kdiv.
//      Flag is 1, if the corresponding div contains low quotient.
//      Initially array of quotients is empty. For the highest quotient
//      the corresponding kdiv is always 0.
//
//      ALGORITHM
//      When the highest quotient is divided quotient is stored in num_
//      and module is stored in div [npmax - 1]. When div [npmax - 1]
//      is divided quotient is stored in div [npmax - 1] and module is
//      stored in div [npmax - 2] e.t.c. Short module and quotient from
//      division of div [0] are coverted to string. While module is
//      converted quotient is stored in div [0]. Then quotient div [1]
//      is moved to div [0] and conversion is repeated, starting from
//      div [0], then div [2] is moved to div [1] and the process is
//      repeated from div [1], e.t.c. np to the lowers quotient, when
//      npmax is decreased and the algorithm is repeated once again.

  size_t j = i;                                 // Position of the 1st digit.
  {
//      Filling of stack of coefficients powradix [radix].
//      Required number of coefficients is returned.

    size_t np = cBigNumberSqrTab (radix, num_.length(), MAX_NP_OUT);
    EXPTR(cBigNumber) powr = EXPTRTYPE(powradix [radix]);

//      Work stacks.

    EXPTR(cBigNumber)  div = EXPTRTYPE(stk_);   // Stack of quotients.
    EXPTR(int)        kdiv = EXPTRTYPE(kstk_);  // Stack of flags.

//      Cycle of quotients.
//      All parts if number are converted, except for the last module

    int numlength = num_.length() / (1024 / sizeof (long));
    if (numlength < MIN_PROGR) numlength = 0;   // Size if KBytes.
    size_t npmax = np;                          // Index of highest quotient.
    for (;;)                                    // Cycle of quotient.
    {
      if (np == npmax)                          // The highest quotient.
      {
        if (numlength > 0)                      // Progress indication.
        {
          cerr << "Formatting"
               << setw (7) << (numlength -
                             num_.length() / (1024 / sizeof (long)))
                                         <<"K of"
               << setw (7) <<  numlength <<"K\b"
               << "\b\b\b\b\b\b\b\b\b\b\b\b\b\b"
               << "\b\b\b\b\b\b\b\b\b\b\b\b\b\b";
        }

//        The highest quotient is divided to the corresponding coefficient.
//        If result is 0 and index of the highest quotient os greater 0
//        then index of the highest quotient decreases and the cycle
//        repeats once again.

        num_.swap (mod_);                       // Store quotient to num_
        num_.setdivmodtab (mod_, powr [np]);    // and module to mod_.

        if (num_.comp0() == 0)                  // Highest quotient is over,
        {                                       // highest module remains.
          if (np == 0) break;                   // If it is the last module
                                                // the cycle is over,
          num_.swap (mod_);                     // else module becomes
          npmax = --np;                         // the highest quotient,
          numlength = -numlength;               // indication os blocked
          assert (kdiv [np] == 0);              // and after debug check
          continue;                             // division is repeated.
        }
      }
      else                                      // Find of quotient.
      {
        assert (np > 0);                        // Debug check.
        assert (kdiv [np - 1] == 0);            // Debug check.
        if (kdiv [np] == 0)                     // If no quotient is found,
        {                                       // then go to higher
          ++np; continue;                       // quotient.
        }                                       // If quotient is found,
        mod_.swap (div [np]);                   // then load div [np]
        kdiv [np] = 0;                          // and mark is as free.
      }

//      Cycle of division of module - which index os greater then 0,
//      divide it to the corresponding coefficient.

      for (; np != 0; np--)
      {
        assert (kdiv [np - 1] == 0);            // Debug check.
        kdiv [np - 1] = 1;                      // Set flag.
        div  [np - 1].setdivmodtab (mod_, powr [np - 1]);
                                                // Store quotient to div
      }                                         // and module to mod_.

//      Cycle of conversion of the lowest module and quotient,
//      which fits to digital grid of type long.
//      The lowest module is stored in mod_ and the lowest quotient,
//      is stored in div [0], if it exists there (it is always so,
//      except for case npmax == 0).

      {
        register long lnum;                     // Integer.
        lnum = mod_.loword();                   // Output of module.
        for (;;)                                // Repeat cycle
        {                                       // once or twice.
          register long knum = maxradix [radix] [1] - 1;
          do                                    // Output knum digits.
          {
            str_ [i++] = ps [ (int) (lnum % radix) ];
            lnum /= radix;                      // Output digits.
          }
          while (--knum != 0);

          assert (lnum >= 0);                   // Debug check.
          assert (lnum < (long) radix);         // Debug check.
          str_ [i++] = ps [(int)(lnum)];        // The last digit.

          if (kdiv [0] == 0) break;             // Is there quotient?
          lnum = div [0].loword();              // Output of quotient.
          kdiv [0] = 0;                         // div [0] is free.
        }
      }

//      Repeat the cycle, starting form the most lower quotient.

      np = (npmax != 0);
    }                                           // Cycle on quotients.

//      Output the last module.

    {
      long lnum = mod_.loword();
      if (lnum != 0)                            // Short module.
      do
      {
        str_ [i++] = ps [(int) (lnum % radix)]; // Output of digit.
      }
      while ((lnum /= radix) != 0);
    }
    if (numlength != 0)                         // Clearing of indicator.
          cerr << "                             \b"
               << "\b\b\b\b\b\b\b\b\b\b\b\b\b\b"
               << "\b\b\b\b\b\b\b\b\b\b\b\b\b\b";

//      Output 0 for number 0.

    if (i == j) str_ [i++] = ps [0];            // The number is 0,
  }

//      Output of fill-in character and finishing of string.

  {
    assert (i > j);                   // Debug check.
    if (flags & ios::internal)        // Format with fill-in character?
    {
      size_t n = os.width();          // Width of field.
      char   c = os.fill();           // Fill-in character.
      while (i < n) str_ [i++] = c;   // Output of fill-in character.
    }
    str_ [i--] = 0;                   // String is ready.
  }

//      Swapping of symbols.

  {
    register EXPTR(char) ps = EXPTRTYPE(str_);

    while (i > j)                       // Swapping of symbols.
    {
      char c = ps [j]; ps [j] = ps [i]; j++; ps [i] = c; i--;
    }
  }

//      Conversion is finished.

  EXDEBUG (num_ = 0);                   // Debug clearing.
  EXDEBUG (mod_ = 0);                   // Debug clearing.
  return ((char *)(const char *)str_);  // Return string.
}

//================================================
//      Operator of output.
//================================================

//      Formatting of output is controlled by the following flags:
//      ios::width()    width of field.
//      ios::fill()     filling symbol.
//      ios::left       align left.
//      ios::right      align right.
//      ios::internal   sign and radix at left, number at right.
//      ios::dec        signed number of radix 10 (default).
//      ios::oct        unsigned number of radix 8.
//      ios::hex        unsigned number of radix 16.
//      ios::showbase   add indicator of radix (0 or 0x).
//      ios::showpos    show sign for positive numbers.
//      ios::uppercase  upper-case letters for number with radix > 10.

ostream& operator << (ostream& os, const cBigNumber& b)
{
  assert (&os);
  assert (&b);
  os << b.toatmp (0, os);
  return os;
}
